package org.xbib.multiserv.plugin.confserv.util;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.net.SocketAddress;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

public class DefaultCommandClient {

    private static final Logger logger = Logger.getLogger(DefaultCommandClient.class.getName());

    public List<String> sendToPort(String host, int port, String cmd) throws IOException {
        logger.log(Level.INFO, "cmd: " + cmd + "\nSLNPQuit\n" );
        List<String> response = new ArrayList<>();
        InetAddress inetAddress = InetAddress.getByName(host);
        SocketAddress socketAddress = new InetSocketAddress(inetAddress, port);
        Socket socket = new Socket();
        try (socket) {
            int connectTimeout = 5 * 1000;
            socket.connect(socketAddress, connectTimeout);
            int readTimeout = 5 * 1000;
            socket.setSoTimeout(readTimeout);
            logger.log(Level.INFO, "client connected to " + socket.getInetAddress() + ":" + socket.getPort());
            InputStream inputStream = socket.getInputStream();
            OutputStream outputStream = socket.getOutputStream();
            BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(inputStream, Charset.defaultCharset()));
            String banner = bufferedReader.readLine();
            logger.log(Level.INFO, "got banner: " + banner);
            outputStream.write((cmd + "\nSLNPQuit\n").getBytes(StandardCharsets.UTF_8));
            outputStream.flush();
            logger.log(Level.INFO, "written: " + cmd + "\nSLNPQuit\n" );
            int attempts = 0;
            while (inputStream.available() == 0 && attempts < 50) {
                attempts++;
                Thread.sleep(200);
            }
            while (inputStream.available() > 0) {
                String s = bufferedReader.readLine();
                logger.log(Level.INFO, "got line: " + s);
                while (s != null && !s.isEmpty() && !s.contains("SLNPEndOfData")) {
                    response.add(s);
                    s = bufferedReader.readLine();
                    logger.log(Level.INFO, "got line: " + s);
                }
            }
            logger.log(Level.INFO, "send to port " + port + " done, response=" + response);
        } catch (Throwable t) {
            logger.log(Level.SEVERE, "SLNPCLient(" + host + ":" + port + "): " + t.getMessage(), t);
        }
        return response;
    }
}
