package org.xbib.multiserv.plugin.confserv.util;

import org.xbib.datastructures.tiny.TinyMap;
import org.xbib.net.http.HttpHeaders;
import org.xbib.net.http.client.ExceptionListener;
import org.xbib.net.http.client.HttpResponse;
import org.xbib.net.http.client.ResponseListener;
import org.xbib.net.http.client.TimeoutListener;
import org.xbib.net.http.client.netty.HttpRequest;
import org.xbib.net.http.client.netty.NettyHttpClient;
import org.xbib.net.http.client.netty.NettyHttpClientConfig;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.function.Consumer;
import java.util.logging.Level;
import java.util.logging.Logger;

public class HttpClientHelper {

    private static final Logger logger = Logger.getLogger(HttpClientHelper.class.getName());

    private final NettyHttpClient client;

    public HttpClientHelper() throws IOException {
        NettyHttpClientConfig config = new NettyHttpClientConfig();
        this.client = NettyHttpClient.builder()
                .setConfig(config)
                .build();
    }

    public void get(String url,
                    Consumer<HttpHeaders> headerCompletion,
                    Consumer<TinyMap.Builder<String, Object>> parameterCompletion,
                    ResponseListener<HttpResponse> responseListener,
                    ExceptionListener exceptionListener,
                    TimeoutListener timeoutListener) throws IOException {
        HttpHeaders httpHeaders = new HttpHeaders();
        if (headerCompletion != null) {
            headerCompletion.accept(httpHeaders);
        }
        TinyMap.Builder<String, Object> parameters = TinyMap.builder();
        if (parameterCompletion != null) {
            parameterCompletion.accept(parameters);
        }
        HttpRequest request = HttpRequest.get()
                .setURL(url)
                .setHeaders(httpHeaders)
                .setParameters(parameters.build())
                .setResponseListener(responseListener)
                .setExceptionListener(exceptionListener)
                .setTimeoutListener(timeoutListener, 15000L)
                .build();
        logger.log(Level.FINE, "executing HTTP GET request " + request);
        client.execute(request).get().close();
    }

    public void post(String url,
                     Consumer<HttpHeaders> headerCompletion,
                     Consumer<TinyMap.Builder<String, Object>> parameterCompletion,
                     String body,
                     ResponseListener<HttpResponse> responseListener,
                     ExceptionListener exceptionListener,
                     TimeoutListener timeoutListener) throws IOException {
        HttpHeaders httpHeaders = new HttpHeaders();
        if (headerCompletion != null) {
            headerCompletion.accept(httpHeaders);
        }
        TinyMap.Builder<String, Object> parameters = TinyMap.builder();
        if (parameterCompletion != null) {
            parameterCompletion.accept(parameters);
        }
        HttpRequest request = HttpRequest.post()
                .setURL(url)
                .setHeaders(httpHeaders)
                .setParameters(parameters.build())
                .setBody(StandardCharsets.UTF_8.encode(body))
                .setResponseListener(responseListener)
                .setExceptionListener(exceptionListener)
                .setTimeoutListener(timeoutListener, 15000L)
                .build();
        logger.log(Level.FINE, "executing HTTP POST request " + request);
        client.execute(request).get().close();
    }
}
